import java.util.Map;
import java.util.TreeMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Collection;
import java.util.Collections;

public class MapOfListDemo {
    
    private static Map<Person, List<Integer>> students = new TreeMap<>(
        new PersonYearOfBirthComparator());
    
    public static void main(String[] args) {
       
       Person honza = new Person("Honza", 2000);
       Person petr = new Person("Petr", 1980);
       Person ales = new Person("Aleš", 1960);
       Person libor = new Person("Libor", 1980);
       
       addMark(honza, 1);
       addMark(ales, 5);
       addMark(ales, 4);
       addMark(ales, 4);
 
       for(Map.Entry<Person, List<Integer>> p: students.entrySet()) {
          Person student = p.getKey();
          Collection<Integer> marks = getMarks(student);
          System.out.print("Student "+student.getName()+" has following marks: ");
          System.out.println(marks);
       }
       
    }
    
    public static void addMark(Person student, int mark) {
        List<Integer> marks = students.get(student);
        if(marks == null) {
            marks = new ArrayList<Integer>();
            students.put(student, marks);
        }
        marks.add(mark);
    }
    
    public static Collection<Integer> getMarks(Person student) {
        List<Integer> marks = students.get(student);
        if(marks == null) {
            marks = new ArrayList<Integer>();
        }
        return Collections.unmodifiableCollection(marks);
    }
        
}
class Person implements Comparable<Person> {
    
    private String name;
    private int yearOfBirth;
    public Person(String name, int yearOfBirth) { 
        this.name = name; 
        this.yearOfBirth = yearOfBirth;
    }
    public String getName() { return name; }
    public void setName(String name) { this.name = name; } 
    
    public int getYearOfBirth() { return yearOfBirth; }
    public void setYearOfBirth(int yearOfBirth) { 
        this.yearOfBirth = yearOfBirth; 
    } 
    
    public boolean equals(Object o) {
        if(o instanceof Person) {
            Person p = (Person)o;
            return this.name.equals(p.name);
        } else return false;
    }
    
    public int hashCode() {
        return name.hashCode();
    }
    
    public int compareTo(Person p) {
        return this.name.compareTo(p.name);
    }
}
class PersonYearOfBirthComparator implements Comparator<Person> {
    public int compare(Person p1, Person p2) {
        int diff = p1.getYearOfBirth() - p2.getYearOfBirth();
        if(diff == 0) {
            return p1.compareTo(p2);
        }
        return diff;
    }
}
class PersonYearOfBirthDescendingComparator extends PersonYearOfBirthComparator {
    public int compare(Person p1, Person p2) {
        return -super.compare(p1, p2);
    }
}